@@ -26,6 +26,7 @@ final class _WP_Editors {
 	private static $editor_buttons_css = true;
 	private static $drag_drop_upload = false;
 	private static $old_dfw_compat = false;
+	private static $translation;
 
 	private function __construct() {}
 
@@ -296,14 +297,13 @@ final class _WP_Editors {
 	/**
 	 * @static
 	 *
-	 * @global string $wp_version
 	 * @global string $tinymce_version
 	 *
 	 * @param string $editor_id
 	 * @param array  $set
 	 */
 	public static function editor_settings($editor_id, $set) {
-		global $wp_version, $tinymce_version;
+		global $tinymce_version;
 
 		if ( empty(self::$first_init) ) {
 			if ( is_admin() ) {
@@ -352,12 +352,19 @@ final class _WP_Editors {
 			if ( empty( self::$first_init ) ) {
 				self::$baseurl = includes_url( 'js/tinymce' );
 
-				$mce_locale = get_locale();
+				$mce_locale = get_user_locale();
 				self::$mce_locale = $mce_locale = empty( $mce_locale ) ? 'en' : strtolower( substr( $mce_locale, 0, 2 ) ); // ISO 639-1
 
 				/** This filter is documented in wp-admin/includes/media.php */
 				$no_captions = (bool) apply_filters( 'disable_captions', '' );
 				$ext_plugins = '';
+				$shortcut_labels = array();
+
+				foreach ( self::get_translation() as $name => $value ) {
+					if ( is_array( $value ) ) {
+						$shortcut_labels[$name] = $value[1];
+					}
+				}
 
 				if ( $set['teeny'] ) {
 
@@ -551,7 +558,8 @@ final class _WP_Editors {
 					'wpeditimage_disable_captions' => $no_captions,
 					'wpeditimage_html5_captions' => current_theme_supports( 'html5', 'caption' ),
 					'plugins' => implode( ',', $plugins ),
-					'wp_lang_attr' => get_bloginfo( 'language' )
+					'wp_lang_attr' => get_bloginfo( 'language' ),
+					'wp_shortcut_labels' => wp_json_encode( $shortcut_labels ),
 				);
 
 				if ( ! empty( $mce_external_plugins ) ) {
@@ -559,7 +567,7 @@ final class _WP_Editors {
 				}
 
 				$suffix = SCRIPT_DEBUG ? '' : '.min';
-				$version = 'ver=' . $wp_version;
+				$version = 'ver=' . get_bloginfo( 'version' );
 				$dashicons = includes_url( "css/dashicons$suffix.css?$version" );
 
 				// WordPress default stylesheet and dashicons
@@ -601,7 +609,7 @@ final class _WP_Editors {
 				$mce_buttons = apply_filters( 'teeny_mce_buttons', array('bold', 'italic', 'underline', 'blockquote', 'strikethrough', 'bullist', 'numlist', 'alignleft', 'aligncenter', 'alignright', 'undo', 'redo', 'link', 'unlink', 'fullscreen'), $editor_id );
 				$mce_buttons_2 = $mce_buttons_3 = $mce_buttons_4 = array();
 			} else {
-				$mce_buttons = array( 'bold', 'italic', 'strikethrough', 'bullist', 'numlist', 'blockquote', 'hr', 'alignleft', 'aligncenter', 'alignright', 'link', 'unlink', 'wp_more', 'spellchecker' );
+				$mce_buttons = array( 'formatselect', 'bold', 'italic', 'bullist', 'numlist', 'blockquote', 'alignleft', 'aligncenter', 'alignright', 'link', 'unlink', 'wp_more', 'spellchecker' );
 
 				if ( ! wp_is_mobile() ) {
 					if ( $set['_content_editor_dfw'] ) {
@@ -623,7 +631,7 @@ final class _WP_Editors {
 				 */
 				$mce_buttons = apply_filters( 'mce_buttons', $mce_buttons, $editor_id );
 
-				$mce_buttons_2 = array( 'formatselect', 'underline', 'alignjustify', 'forecolor', 'pastetext', 'removeformat', 'charmap', 'outdent', 'indent', 'undo', 'redo' );
+				$mce_buttons_2 = array( 'strikethrough', 'hr', 'forecolor', 'pastetext', 'removeformat', 'charmap', 'outdent', 'indent', 'undo', 'redo' );
 
 				if ( ! wp_is_mobile() ) {
 					$mce_buttons_2[] = 'wp_help';
@@ -664,6 +672,7 @@ final class _WP_Editors {
 
 			if ( $post = get_post() ) {
 				$body_class .= ' post-type-' . sanitize_html_class( $post->post_type ) . ' post-status-' . sanitize_html_class( $post->post_status );
+
 				if ( post_type_supports( $post->post_type, 'post-formats' ) ) {
 					$post_format = get_post_format( $post );
 					if ( $post_format && ! is_wp_error( $post_format ) )
@@ -671,9 +680,14 @@ final class _WP_Editors {
 					else
 						$body_class .= ' post-format-standard';
 				}
+
+				if ( $page_template = get_page_template_slug( $post ) ) {
+					$page_template = str_replace( '.', '-', basename( $page_template, '.php' ) );
+					$body_class .= ' page-template-' . sanitize_html_class( $page_template );
+				}
 			}
 
-			$body_class .= ' locale-' . sanitize_html_class( strtolower( str_replace( '_', '-', get_locale() ) ) );
+			$body_class .= ' locale-' . sanitize_html_class( strtolower( str_replace( '_', '-', get_user_locale() ) ) );
 
 			if ( !empty($set['tinymce']['body_class']) ) {
 				$body_class .= ' ' . $set['tinymce']['body_class'];
@@ -807,73 +821,64 @@ final class _WP_Editors {
 		) );
 	}
 
-	/**
-	 * Translates the default TinyMCE strings and returns them as JSON encoded object ready to be loaded with tinymce.addI18n().
-	 * Can be used directly (_WP_Editors::wp_mce_translation()) by passing the same locale as set in the TinyMCE init object.
-	 *
-	 * @static
-	 * @param string $mce_locale The locale used for the editor.
-	 * @param bool $json_only optional Whether to include the JavaScript calls to tinymce.addI18n() and tinymce.ScriptLoader.markDone().
-	 * @return string Translation object, JSON encoded.
-	 */
-	public static function wp_mce_translation( $mce_locale = '', $json_only = false ) {
-
-		$mce_translation = array(
+	private static function get_translation() {
+		if ( empty( self::$translation ) ) {
+			self::$translation = array(
 			// Default TinyMCE strings
 			'New document' => __( 'New document' ),
 			'Formats' => _x( 'Formats', 'TinyMCE' ),
 
 			'Headings' => _x( 'Headings', 'TinyMCE' ),
-			'Heading 1' => __( 'Heading 1' ),
-			'Heading 2' => __( 'Heading 2' ),
-			'Heading 3' => __( 'Heading 3' ),
-			'Heading 4' => __( 'Heading 4' ),
-			'Heading 5' => __( 'Heading 5' ),
-			'Heading 6' => __( 'Heading 6' ),
+			'Heading 1' => array( __( 'Heading 1' ), 'access1' ),
+			'Heading 2' => array( __( 'Heading 2' ), 'access2' ),
+			'Heading 3' => array( __( 'Heading 3' ), 'access3' ),
+			'Heading 4' => array( __( 'Heading 4' ), 'access4' ),
+			'Heading 5' => array( __( 'Heading 5' ), 'access5' ),
+			'Heading 6' => array( __( 'Heading 6' ), 'access6' ),
 
 			/* translators: block tags */
 			'Blocks' => _x( 'Blocks', 'TinyMCE' ),
-			'Paragraph' => __( 'Paragraph' ),
-			'Blockquote' => __( 'Blockquote' ),
+			'Paragraph' => array( __( 'Paragraph' ), 'access7' ),
+			'Blockquote' => array( __( 'Blockquote' ), 'accessQ' ),
 			'Div' => _x( 'Div', 'HTML tag' ),
 			'Pre' => _x( 'Pre', 'HTML tag' ),
 			'Preformatted' => _x( 'Preformatted', 'HTML tag' ),
 			'Address' => _x( 'Address', 'HTML tag' ),
 
 			'Inline' => _x( 'Inline', 'HTML elements' ),
-			'Underline' => __( 'Underline' ),
-			'Strikethrough' => __( 'Strikethrough' ),
+			'Underline' => array( __( 'Underline' ), 'metaU' ),
+			'Strikethrough' => array( __( 'Strikethrough' ), 'accessD' ),
 			'Subscript' => __( 'Subscript' ),
 			'Superscript' => __( 'Superscript' ),
 			'Clear formatting' => __( 'Clear formatting' ),
-			'Bold' => __( 'Bold' ),
-			'Italic' => __( 'Italic' ),
-			'Code' => __( 'Code' ),
+			'Bold' => array( __( 'Bold' ), 'metaB' ),
+			'Italic' => array( __( 'Italic' ), 'metaI' ),
+			'Code' => array( __( 'Code' ), 'accessX' ),
 			'Source code' => __( 'Source code' ),
 			'Font Family' => __( 'Font Family' ),
 			'Font Sizes' => __( 'Font Sizes' ),
 
-			'Align center' => __( 'Align center' ),
-			'Align right' => __( 'Align right' ),
-			'Align left' => __( 'Align left' ),
-			'Justify' => __( 'Justify' ),
+			'Align center' => array( __( 'Align center' ), 'accessC' ),
+			'Align right' => array( __( 'Align right' ), 'accessR' ),
+			'Align left' => array( __( 'Align left' ), 'accessL' ),
+			'Justify' => array( __( 'Justify' ), 'accessJ' ),
 			'Increase indent' => __( 'Increase indent' ),
 			'Decrease indent' => __( 'Decrease indent' ),
 
-			'Cut' => __( 'Cut' ),
-			'Copy' => __( 'Copy' ),
-			'Paste' => __( 'Paste' ),
-			'Select all' => __( 'Select all' ),
-			'Undo' => __( 'Undo' ),
-			'Redo' => __( 'Redo' ),
+			'Cut' => array( __( 'Cut' ), 'metaX' ),
+			'Copy' => array( __( 'Copy' ), 'metaC' ),
+			'Paste' => array( __( 'Paste' ), 'metaV' ),
+			'Select all' => array( __( 'Select all' ), 'metaA' ),
+			'Undo' => array( __( 'Undo' ), 'metaZ' ),
+			'Redo' => array( __( 'Redo' ), 'metaY' ),
 
 			'Ok' => __( 'OK' ),
 			'Cancel' => __( 'Cancel' ),
 			'Close' => __( 'Close' ),
 			'Visual aids' => __( 'Visual aids' ),
 
-			'Bullet list' => __( 'Bulleted list' ),
-			'Numbered list' => __( 'Numbered list' ),
+			'Bullet list' => array( __( 'Bulleted list' ), 'accessU' ),
+			'Numbered list' => array( __( 'Numbered list' ), 'accessO' ),
 			'Square' => _x( 'Square', 'list style' ),
 			'Default' => _x( 'Default', 'list style' ),
 			'Circle' => _x( 'Circle', 'list style' ),
@@ -899,7 +904,7 @@ final class _WP_Editors {
 			'Author' => __( 'Author' ),
 
 			// Media, image plugins
-			'Insert/edit image' => __( 'Insert/edit image' ),
+			'Insert/edit image' => array( __( 'Insert/edit image' ), 'accessM' ),
 			'General' => __( 'General' ),
 			'Advanced' => __( 'Advanced' ),
 			'Source' => __( 'Source' ),
@@ -933,8 +938,8 @@ final class _WP_Editors {
 			'Horizontal line' => __( 'Horizontal line' ),
 			'Horizontal space' => __( 'Horizontal space' ),
 			'Restore last draft' => __( 'Restore last draft' ),
-			'Insert/edit link' => __( 'Insert/edit link' ),
-			'Remove link' => __( 'Remove link' ),
+			'Insert/edit link' => array( __( 'Insert/edit link' ), 'metaK' ),
+			'Remove link' => array( __( 'Remove link' ), 'accessS' ),
 
 			'Color' => __( 'Color' ),
 			'Custom color' => __( 'Custom color' ),
@@ -1038,11 +1043,11 @@ final class _WP_Editors {
 			'Format' => _x( 'Format', 'TinyMCE menu' ),
 
 			// WordPress strings
-			'Toolbar Toggle' => __( 'Toolbar Toggle' ),
-			'Insert Read More tag' => __( 'Insert Read More tag' ),
-			'Insert Page Break tag' => __( 'Insert Page Break tag' ),
+			'Toolbar Toggle' => array( __( 'Toolbar Toggle' ), 'accessZ' ),
+			'Insert Read More tag' => array( __( 'Insert Read More tag' ), 'accessT' ),
+			'Insert Page Break tag' => array( __( 'Insert Page Break tag' ), 'accessP' ),
 			'Read more...' => __( 'Read more...' ), // Title on the placeholder inside the editor (no ellipsis)
-			'Distraction-free writing mode' => __( 'Distraction-free writing mode' ),
+			'Distraction-free writing mode' => array( __( 'Distraction-free writing mode' ), 'accessW' ),
 			'No alignment' => __( 'No alignment' ), // Tooltip for the 'alignnone' button in the image toolbar
 			'Remove' => __( 'Remove' ), // Tooltip for the 'remove' button in the image toolbar
 			'Edit ' => __( 'Edit' ), // Tooltip for the 'edit' button in the image toolbar
@@ -1051,7 +1056,7 @@ final class _WP_Editors {
 			'Link options'  => __( 'Link options' ), // Tooltip for the 'link options' button in the inline link dialog
 
 			// Shortcuts help modal
-			'Keyboard Shortcuts' => __( 'Keyboard Shortcuts' ),
+			'Keyboard Shortcuts' => array( __( 'Keyboard Shortcuts' ), 'accessH' ),
 			'Default shortcuts,' => __( 'Default shortcuts,' ),
 			'Additional shortcuts,' => __( 'Additional shortcuts,' ),
 			'Focus shortcuts:' => __( 'Focus shortcuts:' ),
@@ -1074,7 +1079,8 @@ final class _WP_Editors {
 				__( 'The following formatting shortcuts are replaced when pressing Enter. Press Escape or the Undo button to undo.' ),
 			'The next group of formatting shortcuts are applied as you type or when you insert them around plain text in the same paragraph. Press Escape or the Undo button to undo.' =>
 				__( 'The next group of formatting shortcuts are applied as you type or when you insert them around plain text in the same paragraph. Press Escape or the Undo button to undo.' ),
-		);
+			);
+		}
 
 		/**
 		 * Link plugin (not included):
@@ -1087,10 +1093,31 @@ final class _WP_Editors {
 		 *	Url
 		 */
 
+		return self::$translation;
+	}
+
+	/**
+	 * Translates the default TinyMCE strings and returns them as JSON encoded object ready to be loaded with tinymce.addI18n().
+	 * Can be used directly (_WP_Editors::wp_mce_translation()) by passing the same locale as set in the TinyMCE init object.
+	 *
+	 * @static
+	 * @param string $mce_locale The locale used for the editor.
+	 * @param bool $json_only optional Whether to include the JavaScript calls to tinymce.addI18n() and tinymce.ScriptLoader.markDone().
+	 * @return string Translation object, JSON encoded.
+	 */
+	public static function wp_mce_translation( $mce_locale = '', $json_only = false ) {
 		if ( ! $mce_locale ) {
 			$mce_locale = self::$mce_locale;
 		}
 
+		$mce_translation = self::get_translation();
+		
+		foreach ( $mce_translation as $name => $value ) {
+			if ( is_array( $value ) ) {
+				$mce_translation[$name] = $value[0];
+			}
+		}
+
 		/**
 		 * Filters translated strings prepared for TinyMCE.
 		 *
@@ -1131,13 +1158,12 @@ final class _WP_Editors {
 	/**
 	 *
 	 * @static
-	 * @global string $wp_version
 	 * @global string $tinymce_version
 	 * @global bool   $concatenate_scripts
 	 * @global bool   $compress_scripts
 	 */
 	public static function editor_js() {
-		global $wp_version, $tinymce_version, $concatenate_scripts, $compress_scripts;
+		global $tinymce_version, $concatenate_scripts, $compress_scripts;
 
 		/**
 		 * Filters "tiny_mce_version" is deprecated
@@ -1215,7 +1241,7 @@ final class _WP_Editors {
 
 		$baseurl = self::$baseurl;
 		// Load tinymce.js when running from /src, else load wp-tinymce.js.gz (production) or tinymce.min.js (SCRIPT_DEBUG)
-		$mce_suffix = false !== strpos( $wp_version, '-src' ) ? '' : '.min';
+		$mce_suffix = false !== strpos( get_bloginfo( 'version' ), '-src' ) ? '' : '.min';
 
 		if ( $tmce_on ) {
 			if ( $compressed ) {
