@@ -290,9 +290,9 @@ function wp_print_media_templates() {
 			<div class="thumbnail thumbnail-{{ data.type }}">
 				<# if ( data.uploading ) { #>
 					<div class="media-progress-bar"><div></div></div>
-				<# } else if ( 'image' === data.type && data.sizes && data.sizes.large ) { #>
+				<# } else if ( data.sizes && data.sizes.large ) { #>
 					<img class="details-image" src="{{ data.sizes.large.url }}" draggable="false" alt="" />
-				<# } else if ( 'image' === data.type && data.sizes && data.sizes.full ) { #>
+				<# } else if ( data.sizes && data.sizes.full ) { #>
 					<img class="details-image" src="{{ data.sizes.full.url }}" draggable="false" alt="" />
 				<# } else if ( -1 === jQuery.inArray( data.type, [ 'audio', 'video' ] ) ) { #>
 					<img class="details-image icon" src="{{ data.icon }}" draggable="false" alt="" />
@@ -325,6 +325,8 @@ function wp_print_media_templates() {
 				<div class="attachment-actions">
 					<# if ( 'image' === data.type && ! data.uploading && data.sizes && data.can.save ) { #>
 					<button type="button" class="button edit-attachment"><?php _e( 'Edit Image' ); ?></button>
+					<# } else if ( 'pdf' === data.subtype && data.sizes ) { #>
+					<?php _e( 'Document Preview' ); ?>
 					<# } #>
 				</div>
 			</div>
@@ -454,6 +456,8 @@ function wp_print_media_templates() {
 					<div class="centered">
 						<# if ( data.image && data.image.src && data.image.src !== data.icon ) { #>
 							<img src="{{ data.image.src }}" class="thumbnail" draggable="false" alt="" />
+						<# } else if ( data.sizes && data.sizes.medium ) { #>
+							<img src="{{ data.sizes.medium.url }}" class="thumbnail" draggable="false" alt="" />
 						<# } else { #>
 							<img src="{{ data.icon }}" class="icon" draggable="false" alt="" />
 						<# } #>
@@ -610,16 +614,16 @@ function wp_print_media_templates() {
 					<# } #>>
 
 					<option value="left">
-						<?php esc_attr_e('Left'); ?>
+						<?php esc_html_e( 'Left' ); ?>
 					</option>
 					<option value="center">
-						<?php esc_attr_e('Center'); ?>
+						<?php esc_html_e( 'Center' ); ?>
 					</option>
 					<option value="right">
-						<?php esc_attr_e('Right'); ?>
+						<?php esc_html_e( 'Right' ); ?>
 					</option>
 					<option value="none" selected>
-						<?php esc_attr_e('None'); ?>
+						<?php esc_html_e( 'None' ); ?>
 					</option>
 				</select>
 			</label>
@@ -641,31 +645,31 @@ function wp_print_media_templates() {
 
 				<# if ( data.model.canEmbed ) { #>
 					<option value="embed" selected>
-						<?php esc_attr_e('Embed Media Player'); ?>
+						<?php esc_html_e( 'Embed Media Player' ); ?>
 					</option>
 					<option value="file">
 				<# } else { #>
 					<option value="none" selected>
-						<?php esc_attr_e('None'); ?>
+						<?php esc_html_e( 'None' ); ?>
 					</option>
 					<option value="file">
 				<# } #>
 					<# if ( data.model.canEmbed ) { #>
-						<?php esc_attr_e('Link to Media File'); ?>
+						<?php esc_html_e( 'Link to Media File' ); ?>
 					<# } else { #>
-						<?php esc_attr_e('Media File'); ?>
+						<?php esc_html_e( 'Media File' ); ?>
 					<# } #>
 					</option>
 					<option value="post">
 					<# if ( data.model.canEmbed ) { #>
-						<?php esc_attr_e('Link to Attachment Page'); ?>
+						<?php esc_html_e( 'Link to Attachment Page' ); ?>
 					<# } else { #>
-						<?php esc_attr_e('Attachment Page'); ?>
+						<?php esc_html_e( 'Attachment Page' ); ?>
 					<# } #>
 					</option>
 				<# if ( 'image' === data.type ) { #>
 					<option value="custom">
-						<?php esc_attr_e('Custom URL'); ?>
+						<?php esc_html_e( 'Custom URL' ); ?>
 					</option>
 				<# } #>
 				</select>
@@ -718,13 +722,13 @@ function wp_print_media_templates() {
 				<option value="post" <# if ( ! wp.media.galleryDefaults.link || 'post' == wp.media.galleryDefaults.link ) {
 					#>selected="selected"<# }
 				#>>
-					<?php esc_attr_e('Attachment Page'); ?>
+					<?php esc_html_e( 'Attachment Page' ); ?>
 				</option>
 				<option value="file" <# if ( 'file' == wp.media.galleryDefaults.link ) { #>selected="selected"<# } #>>
-					<?php esc_attr_e('Media File'); ?>
+					<?php esc_html_e( 'Media File' ); ?>
 				</option>
 				<option value="none" <# if ( 'none' == wp.media.galleryDefaults.link ) { #>selected="selected"<# } #>>
-					<?php esc_attr_e('None'); ?>
+					<?php esc_html_e( 'None' ); ?>
 				</option>
 			</select>
 		</label>
@@ -841,16 +845,16 @@ function wp_print_media_templates() {
 			<span><?php _e('Align'); ?></span>
 			<div class="button-group button-large" data-setting="align">
 				<button class="button" value="left">
-					<?php esc_attr_e('Left'); ?>
+					<?php esc_html_e( 'Left' ); ?>
 				</button>
 				<button class="button" value="center">
-					<?php esc_attr_e('Center'); ?>
+					<?php esc_html_e( 'Center' ); ?>
 				</button>
 				<button class="button" value="right">
-					<?php esc_attr_e('Right'); ?>
+					<?php esc_html_e( 'Right' ); ?>
 				</button>
 				<button class="button active" value="none">
-					<?php esc_attr_e('None'); ?>
+					<?php esc_html_e( 'None' ); ?>
 				</button>
 			</div>
 		</div>
@@ -859,13 +863,13 @@ function wp_print_media_templates() {
 			<span><?php _e('Link To'); ?></span>
 			<div class="button-group button-large" data-setting="link">
 				<button class="button" value="file">
-					<?php esc_attr_e('Image URL'); ?>
+					<?php esc_html_e( 'Image URL' ); ?>
 				</button>
 				<button class="button" value="custom">
-					<?php esc_attr_e('Custom URL'); ?>
+					<?php esc_html_e( 'Custom URL' ); ?>
 				</button>
 				<button class="button active" value="none">
-					<?php esc_attr_e('None'); ?>
+					<?php esc_html_e( 'None' ); ?>
 				</button>
 			</div>
 			<input type="text" class="link-to-custom" data-setting="linkUrl" />
@@ -907,16 +911,16 @@ function wp_print_media_templates() {
 						<span><?php _e('Align'); ?></span>
 						<div class="button-group button-large" data-setting="align">
 							<button class="button" value="left">
-								<?php esc_attr_e('Left'); ?>
+								<?php esc_html_e( 'Left' ); ?>
 							</button>
 							<button class="button" value="center">
-								<?php esc_attr_e('Center'); ?>
+								<?php esc_html_e( 'Center' ); ?>
 							</button>
 							<button class="button" value="right">
-								<?php esc_attr_e('Right'); ?>
+								<?php esc_html_e( 'Right' ); ?>
 							</button>
 							<button class="button active" value="none">
-								<?php esc_attr_e('None'); ?>
+								<?php esc_html_e( 'None' ); ?>
 							</button>
 						</div>
 					</div>
@@ -964,21 +968,21 @@ function wp_print_media_templates() {
 						<select data-setting="link">
 						<# if ( data.attachment ) { #>
 							<option value="file">
-								<?php esc_attr_e('Media File'); ?>
+								<?php esc_html_e( 'Media File' ); ?>
 							</option>
 							<option value="post">
-								<?php esc_attr_e('Attachment Page'); ?>
+								<?php esc_html_e( 'Attachment Page' ); ?>
 							</option>
 						<# } else { #>
 							<option value="file">
-								<?php esc_attr_e('Image URL'); ?>
+								<?php esc_html_e( 'Image URL' ); ?>
 							</option>
 						<# } #>
 							<option value="custom">
-								<?php esc_attr_e('Custom URL'); ?>
+								<?php esc_html_e( 'Custom URL' ); ?>
 							</option>
 							<option value="none">
-								<?php esc_attr_e('None'); ?>
+								<?php esc_html_e( 'None' ); ?>
 							</option>
 						</select>
 						<input type="text" class="link-to-custom" data-setting="linkUrl" />
